use strict;
use Test::Simple tests => 5;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError );
use DE_EPAGES::Core::API::File qw ( DeleteFile ExistsFile );
use DE_EPAGES::Core::API::MP3 qw( CutMp3 );
use MP3::Cut::Gapless;

sub Main {

    # CutMp3
    my $StartTime = '00.00';
    my $EndTime = '00.05';

    my $Source = 'MP3/ok.mp3';
    my $Target = 'MP3/ok_cut.mp3';
    eval {
        CutMp3($Source, $Target, $StartTime, $EndTime);
    };
    ok( !ExistsError() && ExistsFile($Target), 'CutMp3: valid mp3 without error' );
    DeleteFile($Target);

    $Source = 'MP3/noid3.mp3';
    $Target = 'MP3/noid3_cut.mp3';
    eval {
        CutMp3($Source, $Target, $StartTime, $EndTime);
    };
    ok( !ExistsError(), 'CutMp3: mp3 without id3 tags without error' );
    DeleteFile($Target);

    # image file renamed to ".mp3"
    $Source = 'MP3/renamedimage.mp3';
    $Target = 'MP3/renamedimage_cut.mp3';
    eval {
        CutMp3($Source, $Target, $StartTime, $EndTime);
    };
    ok( ExistsError(), 'CutMp3: Invalid mp3 file throws error' );
    DeleteFile($Target);

    # mp3 with data junk before first mpeg frame
    $Source = 'MP3/Kalimba.mp3';
    $Target = 'MP3/Kalimba_cut.mp3';
    eval {
        CutMp3($Source, $Target, $StartTime, $EndTime);
    };
    ok( !ExistsError(), 'CutMp3: mp3 with data junk before first mpeg frame cut works' );
    DeleteFile($Target);

    # 56kbit from WinLAME, uses wrong mpeg version header (MPEG2 Layer 3 instead of MPEG1 Layer 3)
    $Source = 'MP3/56k.mp3';
    $Target = 'MP3/56k_cut.mp3';
    eval {
        CutMp3($Source, $Target, $StartTime, $EndTime);
    };
    ok( !ExistsError(), 'CutMp3: 56k.mp3 cut works' );
    DeleteFile($Target);
}

RunScript(Sub=>\&Main);
