#!/bin/sh

DN=$(cd $(dirname $0) && pwd) || exit 1
BN=${0##*/}

usage() {
    echo "Usage:"
    echo "  $BN { -b VERSION | -c CUSTOMIZE/ }"
    echo "Options:"
    echo " -b VERSION: ePages version (e.g. 6.13.1)"
    echo " -c CUSTOMIZE/: directory that contains install.conf and custom scripts"
    echo "              custom scripts are executed if ending in *.sh|*.cmd|*.bat"
    exit 1
}

unset VERSION CUSTOMIZE DOWNLOAD_ONLY CUSTOMIZE_ONLY
while getopts b:c:dn c ; do
  case $c in
    b) VERSION=$OPTARG ;;
    c) CUSTOMIZE=$OPTARG ;;
    d) DOWNLOAD_ONLY=1 ;;
    n) CUSTOMIZE_ONLY=1 ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`
[[ $# -ne 0 ]] && usage
[[ -z $VERSION && -z $CUSTOMIZE ]] && usage
[[ -n $CUSTOMIZE && ! -d $CUSTOMIZE ]] && usage
[[ -n $CUSTOMIZE_ONLY && ! -d $CUSTOMIZE ]] && exit 0

if [[ -z $CUSTOMIZE ]] ; then
  mkdir -m 755 -p /tmp/${BN%.sh}.d || exit 1
  CUSTOMIZE=/tmp/${BN%.sh}.d
fi
INSTALL_CONF="$CUSTOMIZE"/install.conf

# source file

if [[ -f $INSTALL_CONF ]] ; then
  . $INSTALL_CONF
  VERSION=${VERSION:-$EPAGES_INSTALL_VERSION}
fi

RET=0
if [[ -z $CUSTOMIZE_ONLY ]] ; then

  while IFS=. read -a i ; do REL=(${i[@]}) ; done <<<$VERSION

  for i in 0 1 2 ; do
    [[ -z "${REL[$i]//[0-9]/}" ]] || {
      echo "$BN: ePages version ($VERSION): void format"
      usage
    }
  done

  # find/install curl

  if [[ -f /etc/SuSE-release ]] ; then
    INSTALLER=zypper
    ZES=-y ; unset YES
  else
    INSTALLER=yum
    YES=-y ; unset ZES
  fi

  type curl >/dev/null 2>&1 || {
    echo "$INSTALLER $YES install $ZES curl"
    $INSTALLER $YES install $ZES curl
  }

  # rpmify ePages

  rm -f /etc/sysconfig/epages6*
  export EPAGES_INSTALL_VERSION=$VERSION
  curl http://epages.com/RPMS/rpmify/rpmify-epages.sh | sh
  [[ $? = 0 && ${PIPESTATUS[0]} = 0 ]] || exit 1

  [[ -f /etc/SuSE-release ]] && zypper clean -m || yum clean metadata

  if [[ -f $INSTALL_CONF ]] ; then
    cat $INSTALL_CONF >> /etc/sysconfig/epages6
    echo "export EPAGES_INSTALL_VERSION=$VERSION" >> /etc/sysconfig/epages6
  fi
  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6

  # install ePages

  [[ -f /etc/SuSE-release ]] && zypper modifyrepo --enable epages-$VERSION

  if [[ -z $DOWNLOAD_ONLY && -f /etc/SuSE-release ]] ; then
    echo -e "\nzypper install -y -r epages-$VERSION epages\n"
    zypper install -y -r epages-$VERSION epages ; RET=$?
  elif [[ -z $DOWNLOAD_ONLY ]] ; then
    echo -e "\nyum -y install @epages --enablerepo epages-$VERSION\n"
    yum -y install @epages --enablerepo epages-$VERSION ; RET=$?
  elif [[ -f /etc/SuSE-release ]] ; then
    echo -e "\nzypper install -y -r epages-$VERSION epages-appsrv epages-reqsrv epages-mysqld epages-webconf\n"
    zypper install -y -r epages-$VERSION epages-appsrv epages-reqsrv epages-mysqld epages-webconf
    RET=$?
  else
    echo -e "\nyum -y install @epages-appsrv @epages-reqsrv @epages-mysqld @epages-webconf --enablerepo epages-$VERSION\n"
    yum -y install @epages-appsrv @epages-reqsrv @epages-mysqld @epages-webconf --enablerepo epages-$VERSION
    RET=$?
  fi
  [[ -f /etc/SuSE-release ]] && zypper modifyrepo --disable epages-$VERSION

  . /etc/default/epages6
  PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
  ini-config.sh -f /etc/default/epages6 -k LOCAL_IPS -v "" -i
  ini-config.sh -f /etc/default/epages6 -k IP_BINDING -v "" -i
  ini-config.sh -f /etc/yum.repos.d/epages-update.repo -s epages-update -k enabled -v 0 -i

fi # [[ -z $CUSTOMIZE_ONLY ]]

if [[ $RET = 0 && -z $DOWNLOAD_ONLY ]] ; then
  for i in $(ls -d $DN/customize/* 2>/dev/null) ; do
    [[ $i = $DN/customize/install.conf || -d $i ]] && continue
    case $i in
      *.sh|*.cmd|*.bat) EXECUTE=1 ;;
      *) EXECUTE= ;;
    esac
    if [[ -z $EXECUTE || ! -x $i ]] ; then
      echo "ignoring:  $i"
      continue
    fi
    echo "executing: $i"
    $i ; RET=$?
    [ $RET = 0 ] || break
  done
fi

rm -rf /tmp/${BN%.sh}.d
exit $RET
