use strict;
use Test::Simple tests => 6;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::WebService::API::Client;
use Data::Dumper;

sub Main {
    my $IPAddress = '172.20.128.242';
    my $StoreDomainName = 'CentOS242.intern.epages.de';
    my $ShopAlias = 'TestShop';
    my $VMType = 'MerchantStarter';
    my $hHardwareParams = {'VMIpAddress'=>$IPAddress};
    my $soapHWparm = SOAP::Data->name('HardwareParams')->value($hHardwareParams);

    my $Proxy    = 'http://localhost:8080/epages/Site.soap';
    my $User     = '/Providers/Distributor/Users/admin';
    my $Password = 'admin';
    my $FlexProviderService = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/FlexProviderService/2010/07')
        ->proxy($Proxy, timeout => 900 )
        ->userinfo( "$User:$Password" );

    ok( 1, 'FlexProviderService' );

    # configureVM
    my $hContainerParams = {'StoreDomainName'=>$StoreDomainName};
    my $soapCTparm = SOAP::Data->name('ContainerParams')->value($hContainerParams);
    my $ContainerID = $FlexProviderService->configureVM($soapHWparm,$soapCTparm)->result;
    ok( $ContainerID, "configureVM ContainerID=$ContainerID" );

    # createVMshop
    my $hShopParams = {'ShopAlias'=>$ShopAlias};
    my $soapSHparm = SOAP::Data->name('ShopParams')->value($hShopParams);
    my $created = $FlexProviderService->createVMshop($soapHWparm,$soapSHparm)->result;
    ok( $created, 'createVMshop shop created' );

    # getInfoVM
    my $hInfo = $FlexProviderService->getInfoVM($soapHWparm)->result;
    ok( $hInfo->{'ResponseCode'} == 0, 'getInfoVM ResponseCode OK' );
    ok( $hInfo->{'IPAddress'} eq $IPAddress, 'getInfoVM IPAddress OK' );
    print Dumper{'hInfo'=>$hInfo};

    # updateVM
    my $soapVMType = SOAP::Data->name('VMType')->value($VMType);
    my $updated = $FlexProviderService->updateVM($soapHWparm,$soapVMType)->result;
    ok( $updated, 'updateVM' );


}

RunScript( Sub => \&Main );
