use Test::More tests => 3;
use strict;

use DE_EPAGES::Cartridge::API::Table::Cartridge qw (GetPKeyCartridgeByPackage);
use DE_EPAGES::Core::API::File  qw ( GetFileContent DeleteDirectory);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadRootObject);
use DE_EPAGES::Object::API::Language qw (GetCodeByLanguageID);
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw (ExistsPageTypeByAlias LoadPageTypeByAlias);
use DE_EPAGES::Presentation::API::Template qw (ProcessPage);

sub Main {
    ConnectTestSite();
    Cleanup();

    # Add new pagetype for testtemplates.
    my $System = LoadRootObject();
    my $PageTypeClass = LoadClassByAlias( 'PageType' );
    my $TestPageType  = $System->child('PageTypes')->insertChild( $PageTypeClass, {
        'Alias'         => 'TestPageType',
        'PageTypeClass' => LoadClassByAlias( 'System' )
    } );

    # Get cartrdigeid.
    my $CartridgeID = GetPKeyCartridgeByPackage('DE_EPAGES::Presentation');

    # Create a TLEProcessor.
    my $PageType = LoadPageTypeByAlias( 'TestPageType' );
    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load( $PageType, $System );

    # Check GUIObjects.
    foreach my $GUIObject(qw (Accordion Tab Input)){
        # Add testtemplate.
        $TestPageType->insertTemplate( {
            'Name'          => 'GUIHandler-'.$GUIObject,
            'FileName'      => '../t/GUIObject_'.$GUIObject.'_i.html',
            'CartridgeID'   => $CartridgeID
        } );

        # Process template.
        my $Content = ProcessPage(
            $System,
            $ObjectPageType,
            GetCodeByLanguageID(LoadRootObject()->get('LanguageID')),
            {},
            undef,
            undef,
            "GUIHandler-${GUIObject}"
        );

        # Compare content.
        CompareContent($Content, $GUIObject, $GUIObject);
    }

    Cleanup();
    DisconnectSite();
}

sub CompareContent{
    my ($Content, $GUIObject, $Message) = @_;

    # Get filecontent to compare with.
    my $rCompareContent = GetFileContent('GUIObject_'.$GUIObject.'_e.html');
    my $CompareContent = $$rCompareContent;

    # Delete comments, whitespaces and linebreaks.
    $Content =~ s/<!--.*?-->//xsg;
    $Content =~ s/\n//g;
    $Content =~ s/\s//g;
    $CompareContent =~ s/<!--.*?-->//xsg;
    $CompareContent =~ s/\n//g;
    $CompareContent =~ s/\s//g;

    is( $Content, $CompareContent, $Message );
}

sub Cleanup {
    DeleteDirectory( 'ctmpl', { recursive => 1 } );
    LoadPageTypeByAlias( 'TestPageType' )->delete if ExistsPageTypeByAlias( 'TestPageType' );
}

RunScript(
    'Sub' => \&Main
);

1;