jQuery.plugin("de_epages.product.inc.customizeSettings", ["ep.core", "de_epages.core","ep.base64","jQuery.tmpl","jQuery.ui.droppable","jQuery.ui.draggable","jQuery.ui.resizable","jQuery.ui.sortable", "de_epages.presentation.ui.uploader"], function($){

	$.extend( $.scope("de_epages.product.inc"), {
		customizeSettings: function ( productID, selectors, attr ){

			var areaField		= $(selectors.areaInput),

				areaItems		= $(selectors.areas),

				areaHolder		= $(selectors.areaHolder),

				areaBox			= $(selectors.areaBox),

				areaData		= ep.base64Decode(areaField.val() || "e30="),

				areaUpload		= de_epages(selectors.areaUploader),

				areaDrop,

				areaChange = function(){
					areaField.trigger({type:'change',dojoconnect:true});
				},

				areaSave	= function( elem, toggle ){
					var id = elem.data("areaId");

					if( toggle ){
						areaData[ id ] = {
							ProductID	: productID,
							AreaID		: id,
							FromTop		: parseInt( elem.css("top") ),
							FromLeft	: parseInt( elem.css("left") ),
							Width		: elem.width(),
							Height		: elem.height()
						};
					}
					else{
						delete areaData[ id ];
					}

					areaField.val( ep.base64Encode(areaData) );
				},

				areaAdd = function( elem, data ){
					elem.appendTo(areaBox)
						.css({
							position	: "absolute",
							top			: data.FromTop + "px",
							left		: data.FromLeft + "px",
							width		: data.Width + "px",
							height		: data.Height + "px"
						})
						.on("resize",function(){
							areaSave( elem, true );
							areaChange();
						})
						.resizable({
							containment: "parent",
							handles: "n, e, s, w, ne, se, sw, nw"
						});

					$('<a href="#">&#10006;</a>')
						.on("click", function( event ){
							event.preventDefault();
							areaRemove(elem);
						})
						.appendTo(elem);

					areaSave( elem, true );
				},

				areaRemove = function( elem ){
					elem.appendTo(areaHolder)
						.css({
							position: "relative",
							top		: "auto",
							left	: "auto",
							width	: "",
							height	: ""
						})
						.off("resize")
						.empty()
						.resizable("destroy");

					areaSave( elem, false );
					areaChange();
				},

				optionUID		= 0,

				optionField		= $(selectors.optionInput),

				optionData		= ep.base64Decode(optionField.val() || "e30="),

				optionList		= $(selectors.optionList),

				optionItems,

				optionTextButton= $(selectors.optionAddText),

				optionFileButton= $(selectors.optionAddFile),

				optionMax,

				optionChange = function(){
					optionField.trigger({type:'change',dojoconnect:true});
				},

				optionInit = function(){
					optionItems	= optionList.children();

					optionMax = optionItems.length >= 5;

					optionTextButton.attr('disabled',optionMax);
					optionFileButton.attr('disabled',optionMax);
				},

				optionAdd = function( data, initialValidate ){
					if(optionMax){
						return;
					}

					data = typeof data == "string" ? {Type:data,ProductID:productID,OptionID:(optionUID++)} : data;

					$(selectors.optionTemplate)
						.tmpl([ data ])
						.find('.ep-metaparse')
							.metaparse()
						.end()
						.find(selectors.optionRemove)
							.on("click",optionRemove)
						.end()
						.appendTo(optionList)
						.hide()
						.find(':input')
							.each(function(){
								var elem = $(this);
								if( initialValidate ){
									elem.trigger("validate");
								}
								optionSave( elem, data.OptionID );
							})
							.on("change keypress",function(){
								optionSave( $(this), data.OptionID );
								optionChange();
							})
						.end()
						.fadeIn(500);

					optionInit();
				},

				optionRemove = function(event){
					var elem	= $(this),
						item	= elem.closest(selectors.optionList + " > *"),
						id		= item.data("optionId");

					item.remove();

					delete optionData[ id ];
					optionField.val( ep.base64Encode(optionData) );

					optionInit();
					optionChange();
				},

				optionSave = function( elem, id ){

					var data	= optionData[ id ] || (optionData[ id ] = {ProductID:productID,OptionID:id}),
						name	= elem.attr('name').replace(/^Option\d+/,''),
						type	= elem.attr('type'),
						key		= elem.data('optionKey'),
						val;

					if( type == 'checkbox' ){
						val = elem.is(":checked") ? elem.val() : 0;
					}
					else if( type == 'radio'){
						val = elem.is(":checked") ?
							elem.val() :
							elem.closest(selectors.optionList + " > *")
								.find(":radio[name='" + elem.attr("name") + "']:checked")
								.val();
					}
					else {
						val = elem.val();
					}

					// fix number values
					val = $.isNumeric(val) ? parseFloat(val,10) : val;

					// use sub keys
					if( key ){
						(data[ name ] || (data[ name ] = {}))[ key ] = val;
					}
					else{
						data[ name ] = val;
					}

					optionField.val( ep.base64Encode(optionData) );
				},

				previewBox		= $(selectors.previewBox),

				previewOptions	= $(selectors.previewOptions),

				previewOff		= $(selectors.previewOff),

				previewOn		= $(selectors.previewOn),

				previewToggle = function(){
					var show = previewOn.attr('checked');

					previewBox[ show ? 'fadeIn' : 'fadeOut']();
					previewOptions[ show ? 'fadeIn' : 'fadeOut']();
				};

			// init interactive area handles
			areaItems
				.on("dragstop", function( event ){
					var originalEvent = event.originalEvent && event.originalEvent.originalEvent || event.originalEvent,
						container = $(areaDrop),
						elem = $(this);

					if( container.is(selectors.areaHolder)){
						areaRemove( elem );
					}
					else if( !elem.parent().is(selectors.areaBox) ){
						var offset = container.offset();

						areaAdd( elem, {
							FromTop	: (originalEvent.pageY - (originalEvent.offsetY || originalEvent.layerY) - offset.top ),
							FromLeft: (originalEvent.pageX - (originalEvent.offsetX || originalEvent.layerX) - offset.left),
							Width 	: 40,
							Height	: 40
						});
					}
					else{
						areaSave(elem,true);
					}
					areaChange();
				})
				.draggable({
					revert: "invalid",
					connectToSortable: selectors.areaHolder
				});

			areaHolder
				.on("drop",function(){
					areaDrop = this;
				})
				.droppable({
					accept: selectors.areas
				});

			areaBox
				.on("drop",function(){
					areaDrop = this;
				})
				.droppable({
					tolerance: "fit",
					accept: selectors.areas
				});

			if( attr.AreaUploadData ){
				areaUpload
					.find('input:file')
					.presentationUiUploader({
	        			accept:     'image/*',
						data: attr.AreaUploadData,
						callback: function( data ){
							var file;

							if( (file = data && data[0]) && (file = file.success) && (file = file.uploadedFiles) && (file = file[0]) ){
								$('<img/>')
									.attr({
										src: file,
										alt: ''
									})
									.prependTo(areaBox);

								areaUpload.remove();
							}
						}
					});
			}
			else{
				areaUpload.remove();
			}

			// place to saved positions on start up
			areaItems.each(function(){
				var elem	= $(this),
					data	= areaData[ elem.data("areaId") ];

				if( data ){
					areaAdd( elem, data );
				}
			});

			previewOff
				.on("change", previewToggle);

			previewOn
				.on("change", previewToggle)
				.trigger("change");

			// init interactive option handles
			$(document)
        		.on("metaparse", function(){
					optionTextButton
						.on("click",function(){
							optionAdd("text");
							optionChange();
						});

					optionFileButton
						.on("click",function(){
							optionAdd("file");
							optionChange();
						});

					// add saved option on start up
					$.each( ep.hashToArray( optionData, 'key' ).sort(), function( i, key ){
						var data = optionData[ key ];

						// setup the option unique ID
						optionUID = optionUID <= data.OptionID ? data.OptionID+1 : optionUID;
						optionAdd(data, true);
					});

					optionInit();
				});
		}
	});
});