#========================================================================================
# package DE_EPAGES::RemoteSearch::t::ViewRemoteSuggestProducts
# state private
#----------------------------------------------------------------------------------------
# description Tests JSONViewRemoteSuggestProducts with FakeRrequest
#              Needs a Store with DemoShop
#========================================================================================
package DE_EPAGES::RemoteSearch::t::ViewRemoteSuggestProducts;
use base Test::Class;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use DE_EPAGES::WebInterface::API::FakeRequest;
use JSON;
use Test::More;

sub setUp : Test(startup) {
    my $self = shift;
    plan skip_all => 'no demoshop installed' unless ( ExistsObjectByPath('/Shops/DemoShop'));
    plan skip all => 'remote search not enabled' unless (LoadRootObject->get('RemoteSearchEnabled'));

    $self->shop( LoadObjectByPath('/Shops/DemoShop') );
    plan skip_all => 'DemoShop did not initialize remotesearch'
      unless ( defined $self->shop->get('RemoteProductSearchIndex') && $self->shop->get('RemoteProductSearchIndex')->usableAtStorefront );
}

sub TestSimple : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL         = BuildShopUrl( $self->shop, { 'ViewAction' => 'JSONViewRemoteSuggestProducts', 'q' => 'Jack' }, {} );
    my $Response    = $FakeRequest->get($URL);

    ok( $Response->is_success, 'JSON ViewAction works' ) or diag( $Response->status_line );
    my $hSuggestions = decode_json( $Response->decoded_content );
    ok( defined $hSuggestions->{manufacturers}, 'manufacturers' ) or diag( $Response->decoded_content );
    ok( defined $hSuggestions->{categories},    'categories' )    or diag( $Response->decoded_content );
    ok( defined $hSuggestions->{products},      'products' )      or diag( $Response->decoded_content );
}

sub TestEmptyString : Test(no_plan) {
    my $self        = shift;
    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    my $URL         = BuildShopUrl( $self->shop, { 'ViewAction' => 'JSONViewRemoteSuggestProducts' }, {} );
    my $Response    = $FakeRequest->get($URL);

    ok( $Response->is_success, 'JSON ViewAction works' ) or diag( $Response->status_line );
    my $hSuggestions = decode_json( $Response->decoded_content );
    is_deeply( $hSuggestions, {}, 'empty suggestions' );
}

sub shop {
    my $self = shift;
    my ($shop) = @_;
    return $self->{'shop'} unless ( defined($shop) );
    $self->{'shop'} = $shop;
}

RunScript(
    Sub => sub {
        RunOnStore(
            'Store' => GetTestSite(),
            'Sub'   => sub {
                Test::Class->runtests;
            }
        );
    }
);
